# bldcobol.pl - Compile a COBOL file
# 
# 
#
# Licensed Materials - Property of IBM
# Restricted Materials of IBM
# Rational ClearCase
# (C) Copyright IBM Corp. 2006, 2014.  All Rights Reserved
# US Government Users Restricted Rights 
# Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
# 
# 
# The common routines used in all the build scripts are contained in the 
# bccpbmod.pm module.
#
# This script is spawned from the bccPmbld_main.pl script for each COBOL
# file that was selected to be compiled.  
#
# The calling syntax for this script is:
# 	Usage: bldcobol.pl [-x] [-1] -d directory_path -f filename
# Where;
# 	-x turns on the Expeditor option in the compiler
# 	-1 indicates that the compilation list is one file which causes
# 	   the log output to be displayed to stdout as well as the log file
# 	-d identifies the directory where the script and logs exist
# 	-f identifies the file to be compiled
#
# This script can be limited to specific users during development by using the
# following lines and changing the username strings.
# In this example an user id table 'userid' is used and queried from bccpbmod.pm.
# If using your site security system, this step may be skipped.
#
# my $user = $ENV{'USERNAME'};
# exit 0 if ((lc $user ne "USER1") && (lc $user ne "USER2"));
#
# You must modify this script before attempting to use parameterized build.
# Sections that require modification and the instructions for modifying them
# are flagged with the following comment lines:
#
# ******************************************************************************
# * Customization instructions:                                                *
# * [instructions here]                                                        *
# ****************************************************************************** 

use File::Basename;
use File::Copy;
use POSIX qw(strftime);
use Getopt::Std;
use bccpbmod;

my $debug      = $bccpbmod::debug;     # for verbose debugging output
my $debug      = 1;                    # for verbose debugging output

# Setting verbosity level in this example it is set to be full thorugh out
# the build process this will display all output at the fullest level
# 
my $rccVerbosity = "";
$rccVerbosity = '-V -V -V' if $debug;

# Setting for all output from each step
# this is used in each step of the compile process for all output
#
my $sysout       = $bccpbmod::sysout;
$sysout = 'RCCEXT=RCCOUT,DISP=(NEW,CATLG,DELETE),' if $debug;
my $sysout1 = 'SPACE=(32000,(30,30))' if $debug;
my $jcl;	    # JCL filename
my $ncpycnt = 7;    # number of CICS NewCopy steps if CICS new copy is used, this is an example
my $lcnt    = 0;    # general loop counter

# #############################################################################
# process input arguments
#
#
my $usage = "bldcobol [-x] [-1] -d dir -f file";

my $exped_opt = 0; # Xpediter option
my $compile_1 = 0; # Compilation from main consisted of one file
my $dir_file  = ''; # directory where the file resides
my $filename  = ''; # filename - full or relative path

if (!getopts('x1d:f:') || $opt_h) {
    print "$usage \n";
    exit(0);
}

$exped_opt = $opt_x;
$compile_1 = $opt_1;
$dir_file  = $opt_d;
$filename  = $opt_f;

my $current_dir = `cd`;
print "Build script: current dir =  $current_dir \n" if $debug;

print "Build script: One file = $compile_1 \n" if $debug;
print "Build script: Xpediter = $exped_opt \n" if $debug;
print "Build script: filename = $filename \n" if $debug;
print "Build script: dir_file = $dir_file \n" if $debug;


# Retrieve the environment information from the CC Project attribute
#
# *******************************************************************************
# * Optional customization:                                                     *
# * You can use the optional variables or add new variables based on your site's*
# * needs.  If you determine to use the optional variables or add new variables,*
# * you have to update all related scripts correspondently.                     *
# *******************************************************************************
my ($env, $db2sys, $menv, $os390_hostname, $os390_port, $stage,
    $copylib1, $copylib2, $copylib3, $copylib4,
    $syslib1, $syslib2, $syslib3,
#   $syslib4, $steplib1, $steplib2,			 # optional
#   $hostpds_prefix, $hostpds_affix, $cbllib,	         # optional 
    $loadlib, $c390llib, $dbrmlib, $tgtlib, $db2_owner) =
  bccpbmod::setEnvironmentVar();

# print "Env: $env \n" if $debug;
# print "db2sys: $db2sys \n" if $debug;

# #############################################################################
# Translate the distributed username to a mainframe ID
# In this example the distributed userid is translated to the Z/OS id
# this is done for the following reasons
# 1. To use as a devprint to track who compiled the element and waht version
#    this will be used and installed in the load modules IDRDATA
# 2. To track and route output back to the ClearCase view
# 3. For cleanup purposes after the build process is completed in the 
#    context of the user's view
# 4. For security purposes for the creation of TEMP work files on Z/OS
#    as part of the build process
#
# Please note that the started task userid is the only userid that can update files
# on Z/OS care should be taken when defining TEMP works file. It is recommended that a 
# genral HLQ be used, the started task will gain access to that HLQ than the user specific
# information be added after that HLQ. For example
#
# HLQ.username... 
#        
my $pcid = $ENV{USERNAME};
my $mfid = bccpbmod::getUserID($pcid);
bccpbmod::my_die('Unknown User.') if ($mfid eq "");

# #############################################################################
# Set the local copylib and jcl/log directory for the current user
#
# #############################################################################
# *******************************************************************************
# * Optional customization:                                                     *
# * You may hardcode PDS with a prefix or an affix.                             *
# *******************************************************************************
#my $name = "MFX." . $mfid;       # hardcode PDS with a  prefix => MFX.USERID
my $name = $mfid . ".A";          # hardcode PDS with an affix  => USERID.A

# *******************************************************************************
# * Optional customization:                                                     *
# * The following code can be used to generate the PDS if the type attribute    *
# * ZCCENV is set with the $hostpds_prefix and $hostpds_affix.                  *
# *******************************************************************************          
#
#  my $name = $mfid;
#  my $affix ="";
#  my $prefix ="";
#  if (($hostpds_prefix ne '') || ($hostpds_affix ne '')) {
#     if ($hostpds_prefix ne '') {
#        $prefix = $hostpds_prefix . ".";
#     }
#     if ($hostpds_affix ne '') {
#        $affix = "." . $hostpds_affix ;
#     }
#     $name = $prefix . $mfid . $affix ;
#   } 
# #############################################################################


print "Build script: PCID: $pcid, MFID: $mfid, NAME: $name\n" if $debug;


# #############################################################################
# determine the member name from the clearcase filename. The member name is
# just the filename without the front path and extension.
#
my $delimiter= '\\';
my $del_ind  = rindex( $filename, $delimiter);
my $mem_ext  = substr( $filename, ($del_ind == -1 ? 0: ($del_ind+1)), length($filename));
my $dot      = rindex( $mem_ext, '.' );
my $mem      = uc( substr( $mem_ext, 0, $dot ) );
my $ext      = substr( $mem_ext, $dot );

my $jcldir = $dir_file.$delimiter.$mfid.$delimiter.$mem;

printf( "Build script: processing file (%s)\n", $filename ) if $debug;
printf( "Build script:  mem (%s) ext (%s)\n", $mem, $ext ) if $debug;

# #############################################################################
# Create identifier string for current file
# This is the identifier that will be added to the IDRDATA of the compiled
# Load module.
# In this example enviroment name, element name, and mainframe userid,
# and filename with full extention is used, this can be site specific
#
# *******************************************************************************
# * Optional customization:                                                     *
# * Modify this based on the needs.                                             *
# ******************************************************************************* 
# my $identifier = bccpbmod::devPrint($env,$mem,$mfid,$filename);   
my $identifier = 'devPrint returns length too long';                # test 

# #############################################################################
# Retrieve file information from element attributes
#
my %CCattr = bccpbmod::getCCattrs("..\\..\\$filename");

while (($attr, $value) = each %CCattr) {
	print "Attribute: $attr\tValue: $value\n";
	if ($compile_1) {
		print STDERR "Attribute: $attr\tValue: $value\n";
	}
}

# *******************************************************************************
# * Required customization:                                                     *
# * The following code has to be modified to meet your site's requirement.      *
# *******************************************************************************
# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< main >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#
# The main routine continues by calling each of the specific job routines.
# If any of the routines fail, this scripts exits with an error message.
#
#
bccpbmod::my_die('Build error.')                if ( build() != 0 );

# Otherwise, exit successfully
exit 0;

##############################################################
# The following subroutines contain each of the JCL decks
# that execute specific mainframe tasks
#
# Each subroutine performs the following functions:
#    1) build JCL file name
#    2) write specific JCL to the file
#    3) execute the JCL
#    4) report status
#
##############################################################
#
#
# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< build >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#
#
sub build {
    $jcl = 'build_' . $mem .'.jcl';
    $jcl = $jcldir . $delimiter . $jcl;
    if(-f $jcl)
    {
    	unlink($jcl);
    }
##############################################################
# This is just an example specific site standards and naming conventions
# must be used in addition to TEMP datasets to be used, this step allocates
# the TEMP work datasets to be used.
# ddnames are: the $name varible has the following format
#
# UPLD1 - HLQ.userid.SOURCE datasets that holds elements to be compiled that 
#         where part of the user request to compile, this source is sent
#         from the ClearCase repository to Z/OS 
# TEMP1 - HLQ.userid.OBJECT dataset name that holds object file from compile
# TEMP2 - HLQ.userid.LISTING dataset name that holds the listing from compile
# TEMP3 - HLQ.userid.copylibs dataset that holds any modified copybooks
# TEMP4 - HLQ.userid.LDPREP datasets that holds load module in temp file to be packaged
#         and sent to the CC repsitory for deployment later.
#         
# TEMP5 - HLQ.userid.DBRMLIB datasets that holds any modified DBRM's
# TEMP6 - HLQ.userid.DB2PC datasets that holds DB2 pre-compile
# TEMP7 - HLQ.userid.LOADNAME datasets used to add IDRDATA to load module 
# TEMP8 - HLQ.userid.HECPOUT dataset used to hold CICS translation 
# 
# The following datasets will be kept for the build process untill manually delete
# TEMPA - HLQ.userid.XLOADLIB datasets used to hold load module. 
# TEMPB - HLQ.userid.XOBJLIB  dataset used to hold object file from compile.   
# TEMPC - HLQ.userid.XCOPYLIB dataset used to hold any modified copybooks.
#
    bccpbmod::writeFile(
        $jcl, $compile_1, "
//* ------------------------------------------------------------------
//S1ALLOC  EXEC PGM=IEFBR14
//*        ----
//*        ---- IEFBR14 - Upload and Allocate Temporary Files
//*        ----
//UPLD1   DD  DSN=$name.SOURCE,RCCEXT=CBL,
//             DISP=OLD
//TEMPA    DD  DSN=$name.XLOADLIB,
//             DISP=(MOD,KEEP,DELETE),
//             UNIT=SYSDA,SPACE=(CYL,(5,5,5)),
//             DCB=(RECFM=FB,LRECL=80,BLKSIZE=3200)
//TEMPB    DD  DSN=$name.XOBJLIB,
//             DISP=(MOD,KEEP,DELETE),
//             UNIT=SYSDA,SPACE=(CYL,(5,5,5)),
//             DCB=(RECFM=FB,LRECL=80,BLKSIZE=3200)
//TEMPC    DD  DSN=$name.XCOPYLIB,
//             DISP=(MOD,KEEP,DELETE),
//             UNIT=SYSDA,SPACE=(CYL,(5,5,5)),
//             DCB=(RECFM=FB,LRECL=80,BLKSIZE=3200)
//* ------------------------------------------------------------------
"
    );

##############################################################    
# This is just an example specific site standards and naming conventions
# must be used in addition naming convention for system datasets related to CICS
# must also be specified and our site specific. In addition PARM statements are also
# site specific they can be substituted in via a varible and can also be element specific
# based on the element level attributes assigned.
#
# The first check is to verify that the element does have a CICS componant this is determined by
# attributes assigned to the element if the CICS flag is equal to Y then this step will execute
# otherwise this step will be skipped and the compile process will continue.
# 
# the dataset naming convention are:
# ddnames are: the $name varible has the following format
# HLQ.userid.*
#
# STEPLIB - Load library of CICS modules this must be changed to site standard 
# SYSIN - TEMP file that was allocated above stores source to be compiled
# SYSPRINT - Will send translation output back to repository in the context of the users CC view
#            please see the $sysout varible that was set at the begining of this build engine.
# SYSPUNCH - Dataset that was allocated above to hold CICS translation output
# 
    if ( $CCattr{'CICS'} eq 'Y' ) {
        bccpbmod::writeFile(
            $jcl, $compile_1, "
//* ------------------------------------------------------------------
//CICS     EXEC PGM=DFHECP1\$,
//*        ----
//*        ---- DFHECP1\$ - CICS Translator
//*        ----
//   PARM='APOST,COBOL3,NUM,OP,SOURCE,VBREF,SP'
//STEPLIB  DD  DISP=SHR,DSN=SYS2.CICSTS13.SDFHLOAD
//SYSIN    DD  DISP=OLD,DSN=$name.SOURCE($mem)
//SYSPRINT DD  $sysout
//             $sysout1 
//SYSPUNCH DD  DISP=SHR,DSN=$name.HECPOUT
//* ------------------------------------------------------------------
"
        );
	}

##############################################################    
# This is just an example specific site standards and naming conventions
# must be used in addition naming convention for system datasets related to DB2
# must also be specified and our site specific. In addition PARM statements are also
# site specific they can be substituted in via a varible and can also be element specific
# based on the element level attributes assigned.
#
# The first check is to verify that the element does have a DB2 componant this is determined by
# attributes assigned to the element if the DB2 flag is equal to Y then this step will execute
# otherwise this step will be skipped and the compile process will continue.
#
# In addition the SYSLIB concatenation of this step is determined by where the compile process is
# ocurring, this is done by naming convention in this example, If the CC stream structure is named TEST, 
# QA, and PROD then the appropriate concatenation will be selected based on where in the Life cycle the 
# compilation is occuring. The CC stream structure maps to the corrosponding Z/OS infrastructure or 
# Environment.
#
#
# 
# the dataset naming convention are:
# ddnames are: the $name varible has the following format
# HLQ.userid.*
#
# STEPLIB - Load library of DB2 modules this must be changed to site standard 
# SYSIN - TEMP file that was allocated above stores source to be compiled
# SYSPRINT - Will send translation output back to repository in the context of the users CC view
#            please see the $sysout varible that was set at the begining of this build engine.
# SYSCIN - Dataset that was allocated above to hold CICS translation output
# SYSLIB - The concatenation based on environment to resolve copybook, DCLGEN's
# SYSTERM - work file required by DB2
# SYSUDUMP - dump file required by DB2
# SYSUT1 - work file required by DB2
# SYSUT2 - work file required by DB2
#
#
# Please note that in this example it is a straight DB2 COBOL batch compile if the COBOL program
# had a CICS build componant then an IF statement would need to be added to accomdate the CICS translation
# source to be read in as SYSIN, Please look at the SYSLIB concatenation example, the CICS attribute would
# need to be tested for Y or N, IF Y then the $name.HECPOUT dataset would need to be read in for the SYSIN step   
#
    if ( $CCattr{'DB2'} eq 'Y' ) {
        bccpbmod::writeFile(
            $jcl, $compile_1, "
//* ********************************************  DB2 COBOL PRE COMPILE
//DB2PC    EXEC PGM=DSNHPC,
//  PARM=('SOURCE,DATE(USA),HOST(COBOL),APOST,SOURCE')
//STEPLIB  DD  DSN=DB2.TEST.DSNLOAD,DISP=SHR
//DBRMLIB  DD  DSN=$name.DBRMLIB($mem),DISP=OLD
//SYSIN    DD  DSN=$name.SOURCE($mem),DISP=OLD
//SYSCIN   DD  DSN=$name.DB2PC,DISP=SHR
"
        );


        if ( $env eq $bccpbmod::PROD_env ) {
# PROD
            bccpbmod::writeFile( $jcl, $compile_1, "
//SYSLIB   DD DSN=$copylib1,DISP=SHR
"
            );
        } elsif ( $env eq $bccpbmod::QA_env ) {
# QA
            bccpbmod::writeFile( $jcl, $compile_1, "
//SYSLIB   DD DSN=$copylib1,DISP=SHR
//         DD DSN=$copylib2,DISP=SHR
//         DD DSN=$copylib3,DISP=SHR
"
            );
        } elsif ( $env eq $bccpbmod::DEV_env ) {
# DEV
            bccpbmod::writeFile( $jcl, $compile_1, "
//SYSLIB   DD DSN=$copylib1,DISP=SHR
//         DD DSN=$copylib2,DISP=SHR
//         DD DSN=$copylib3,DISP=SHR
//         DD DSN=$copylib4,DISP=SHR
"
            );
        }


        bccpbmod::writeFile(
            $jcl, $compile_1, "
//SYSPRINT DD  $sysout
//             $sysout1
//SYSTERM  DD  $sysout
//             $sysout1
//SYSUDUMP DD  SYSOUT=*
//SYSUT1   DD  SPACE=(800,(500,500),,,ROUND),UNIT=VIO
//SYSUT2   DD  SPACE=(800,(500,500),,,ROUND),UNIT=VIO
//* **********
"
        );
    }

##############################################################
# This is just an example specific site standards and naming conventions
# must be used in addition naming convention for system datasets related to DB2,CICS
# must also be specified and our site specific. In addition PARM statements are also
# site specific they can be substituted in via a varible and can also be element specific
# based on the element level attributes assigned In this example a varible is used.
#
# The first check is to verify the Xpeditor option, If this is specified the PJCL
# that is created will take a different path and also execute a differenr compile program
# This is just an example but similiar logic would be used fro ABENDAID or IBM DEDUG tools.
#
# the next check is ti verify that the elment does not have any elemnet specific parms or not, if it does those
# parameters are used if not the defualt is used.
#
# The next check is to verify if there is a DB2 component, this will determine what the input file name is
# for the compile process, Plese note that in this example DB2 is checked only if CICS or CICS and DB2 where 
# build components then the input file would be different. the logic provided can be used as a guide. In this example
# DB2 attribute is checked if it is a Y then the DB2 output file from the DB2 pre-compile will be used, otherwise it is
# just the source file.   
#
# 
# In addition the SYSLIB concatenation of this step is determined by where the compile process is
# ocurring, this is done by naming convention in this example, If the CC stream structure is named TEST, 
# QA, and PROD then the appropriate concatenation will be selected based on where in the Life cycle the 
# compilation is occuring. The CC stream structure maps to the corrosponding Z/OS infrastructure or 
# Environment.
#
# The next phase is the compile itself based on if the Xpediter option was selected or not via the 
# context menu in CC or Windows explorer.
# 
# the dataset naming convention are:
# ddnames are: the $name varible has the following format
# HLQ.userid.*
#
# 
# SYSIN - TEMP file that was allocated above stores source to be compiled
# SYSPRINT - Will send translation output back to repository in the context of the users CC view
#            please see the $sysout varible that was set at the begining of this build engine.
# SYSLIB - The concatenation based on environment to resolve copybook, DCLGEN's
# SYSLIN - dataset that will store the OBJ file to be passed to the LINK step
# SYSUDUMP - dump file required by compile process
# SYSUT1 - work file required by compile process
# SYSUT2 - work file required by compile process
# SYSUT3 - work file required by compile process
# SYSUT4 - work file required by compile process
# SYSUT5 - work file required by compile process
# SYSUT6 - work file required by compile process
# SYSUT7 - work file required by compile process
#
#
# Please note that in this example it is a straight DB2 COBOL batch compile if the COBOL program
# had a CICS build componant then an IF statement would need to be added to accomdate the CICS translation
# source to be read in as SYSIN, Please look at this example as a guide to accomplish this, the CICS attribute would
# need to be tested for Y or N, IF Y then the $name.HECPOUT dataset would need to be read in for the SYSIN step,
# In addition to pointing to different libraries that contained the CICS load modules.   
#
     bccpbmod::writeFile(
         $jcl, $compile_1, "
//* *****************************************  COMPILE:
//* ***********************************   BSA
//* COBOL390 = Y
//*
"
     );

     if ($exped_opt) {
	 # Expedite option set
         bccpbmod::writeFile(
             $jcl, $compile_1, "
//COB      EXEC PGM=CWPCMAIN,
"
         );
     } else {
	 # No Expedite option set
         bccpbmod::writeFile(
             $jcl, $compile_1, "
//COB      EXEC PGM=IGYCRCTL,
"
         );
     }

     if ( $CCattr{'COPT'} eq 'N' ) {
         # no COPT value
 bccpbmod::writeFile( $jcl, $compile_1, "
//  PARM=('S')
"
         );
     } else {
         # use COPT value in PARM
         bccpbmod::writeFile( $jcl, $compile_1, "
//  PARM=('S,LIB,$CCattr{'COPT'}')
"
         );
     }

     if ( $CCattr{'DB2'} eq 'Y' ) {

         bccpbmod::writeFile(
             $jcl, $compile_1, "
//SYSIN    DD DSN=$name.DB2PC,DISP=SHR
"
         );
     } else {
         bccpbmod::writeFile(
             $jcl, $compile_1, "
//SYSIN    DD DSN=$name.SOURCE($mem),DISP=OLD
"
         );
     }


    if ( $env eq $bccpbmod::PROD_env ) {
# PROD
        bccpbmod::writeFile( $jcl, $compile_1, "
//SYSLIB   DD DSN=$copylib1,DISP=SHR
"
        );
    } elsif ( $env eq $bccpbmod::QA_env ) {
# QA
        bccpbmod::writeFile( $jcl, $compile_1, "
//SYSLIB   DD DSN=$copylib1,DISP=SHR
//         DD DSN=$copylib2,DISP=SHR
//         DD DSN=$copylib3,DISP=SHR
"
        );
    } elsif ( $env eq $bccpbmod::DEV_env ) {
# DEV
             bccpbmod::writeFile( $jcl, $compile_1, "
//SYSLIB   DD DSN=$name.XCOPYLIB,DISP=SHR
//         DD DSN=$copylib1,DISP=SHR
//         DD DSN=$copylib2,DISP=SHR
//         DD DSN=$copylib3,DISP=SHR
//         DD DSN=$copylib4,DISP=SHR
"
        );
     }

     bccpbmod::writeFile( $jcl, $compile_1, "
//*SYSLIN   DD DSN=$name.OBJECT,DISP=SHR 
//SYSLIN   DD DSN=$name.XOBJLIB($mem),DISP=OLD
//SYSPRINT DD $sysout
//            $sysout1 
//SYSUDUMP DD SYSOUT=*
//SYSUT1   DD UNIT=VIO,SPACE=(CYL,(4,2))
//SYSUT2   DD UNIT=VIO,SPACE=(CYL,(1,1))
//SYSUT3   DD UNIT=VIO,SPACE=(CYL,(1,1))
//SYSUT4   DD UNIT=VIO,SPACE=(CYL,(1,1))
//SYSUT5   DD UNIT=VIO,SPACE=(CYL,(1,1))
//SYSUT6   DD UNIT=VIO,SPACE=(CYL,(1,1))
//SYSUT7   DD UNIT=VIO,SPACE=(CYL,(4,2))

");

     if ($exped_opt) {
	 # Expedite option set
         bccpbmod::writeFile(
             $jcl, $compile_1, "
//CWPDDIO  DD DSN=COMPWARE.LISTING.FILE02,DISP=SHR
//CWPPRMO  DD *
LANGUAGE(VSCOBOLII)
COBOL(OUTPUT(NOPRINT,NODDIO))
PROCESSOR(OUTPUT(PRINT,DDIO))
PRINT(OUTPUT(NOXREF,NODMAP,NOMAP,NOLIST,NOOFFSET))
DDIO(OUTPUT(NOXREF,NODMAP,NOPMAP,NOLIST,NOCLIST,NOOFFSET))
DDIO(OUTPUT(FIND))
SIZE(3000K)
"
         );
     }
     
##############################################################
#
# This is just an example specific site standards and naming conventions
# must be used in addition naming convention for system datasets related to DB2,CICS
# MQ Series etc must also be specified and our site specific. In addition PARM statements are also
# site specific they can be substituted in via a varible and can also be element specific
# based on the element level attributes assigned In this example a varible is used.
#
#
#this routine creates a Link control card to be used to populate the IDRDATA
#                    feild within the load module it is input to the link steps
    bccpbmod::writeFile( $jcl, $compile_1, "
//* ------------------------------------------------------------------
//LDNAME   EXEC PGM=IEBGENER
//*        ----
//*        ---- IEBGENER - Create Loadname entry
//*        ----
//*SYSUT1   DD  *
//*  IDENTIFY $mem('$identifier')  
//*  NAME $mem(R) 
//SYSUT1   DD  *
  INCLUDE OBJLIB($mem)
  NAME $mem(R)
/*
//SYSUT2   DD  DISP=OLD,DSN=$name.XLOADLIB($mem)
//SYSPRINT DD  $sysout
//             $sysout1
//SYSIN    DD  DUMMY
//* ------------------------------------------------------------------
"
    );
    
##############################################################
# This is just an example specific site standards and naming conventions
# must be used in addition naming convention for system datasets related to DB2,CICS,MQ
# must also be specified and our site specific. In addition PARM statements are also
# site specific they can be substituted in via a varible and can also be element specific
# based on the element level attributes assigned In this example a varible is used.
#
# This routine will Link edit an element, this example will linkedt a Cobol bacth program
# a CICS program and all combination of elements with build components that have DB2, CICS, MQ series
#
# The First check is ti verify that the element does not have any elemnet specific parms or not, if it does those
# parameters are used if not the defualt is used.
#
# The next check is to verify if there where the element is being compiled, TEST, QA, PROD and the appropriate SYSLIB
# concatenation is selected for the environment, then element level components such as DB2, CICS and MQ are determined and the 
# appropraite system datasets are added to the SYSLIB concatenation, Target loadlibs and subsystems are also determined
# 
# 
# In addition the SYSLIB concatenation of this step is determined by where the Link process is
# ocurring, this is done by naming convention in this example, If the CC stream structure is named TEST, 
# QA, and PROD then the appropriate concatenation will be selected based on where in the Life cycle the 
# compilation is occuring. The CC stream structure maps to the corrosponding Z/OS infrastructure or 
# Environment.
#
# The Link process is then run 
#  
# the dataset naming convention are:
# ddnames are: the $name varible has the following format
# HLQ.userid.*
#
    my $lopt_parm = 'SIZE=(512K,96K)';
    if ( $CCattr{'LOPT'} ne 'N' ) {
        $lopt_parm  = $CCattr{'LOPT'};
    }

    bccpbmod::writeFile( $jcl, $compile_1, "
//* *********************************************  LINK-EDIT THE PROGRAM
//* COBOL390 = Y
//LKED     EXEC PGM=IEWL,
//  PARM='$lopt_parm'
//OBJLIB   DD DSN=$name.XOBJLIB($mem),DISP=OLD
//SYSLIN   DD DSN=$name.XLOADLIB($mem),DISP=OLD
"
    );

    if ( $env eq $bccpbmod::PROD_env) {
# PROD
        bccpbmod::writeFile( $jcl, $compile_1, "
//SYSLIB   DD DSN=$syslib1,DISP=SHR
//         DD DSN=CEE.SCEELKED,DISP=SHR
//         DD DSN=UTL.PGMLIB,DISP=SHR
"
         );
    } elsif ( $env eq $bccpbmod::QA_env) {
# QA
         bccpbmod::writeFile( $jcl, $compile_1, "
//SYSLIB   DD DSN=$syslib1,DISP=SHR
//         DD DSN=$syslib2,DISP=SHR
//         DD DSN=CEE.SCEELKED,DISP=SHR
//         DD DSN=UTL.PGMLIB,DISP=SHR
"
         );
    } elsif ( $env eq $bccpbmod::DEV_env) {
# DEV
         bccpbmod::writeFile( $jcl, $compile_1, "
//SYSLIB   DD DSN=$syslib1,DISP=SHR
//         DD DSN=$syslib2,DISP=SHR
//         DD DSN=$syslib3,DISP=SHR
//*        DD DSN=CEE.SCEELKED,DISP=SHR
//         DD DSN=SYS1.SCEELKED,DISP=SHR
//*        DD DSN=UTL.PGMLIB,DISP=SHR
//*        DD DSN=DB2.TEST.DSNLOAD,DISP=SHR
"
         );
     }


     # check IMS/DL1
     if ( $CCattr{'DL1'} eq 'Y' ) {
         bccpbmod::writeFile( $jcl, $compile_1, "
//         DD DSN=IMS.RESLIB.TEST,DISP=SHR
//RESLIB   DD DSN=IMS.RESLIB.TEST,DISP=SHR
"
         );
     }


        if ( $CCattr{'CICS'} eq 'Y' ) {
            bccpbmod::writeFile(
                $jcl, $compile_1, "
//          DD  DISP=SHR,DSN=CICSTS13.SDFHEXCI
"
            );
        }

        if ( $CCattr{'MQ'} eq 'Y' ) {
            bccpbmod::writeFile(
                $jcl, $compile_1, "
//          DD  DISP=SHR,DSN=MQSERIES.SCSQLOAD
"
            );
        }


        if ( $CCattr{'DB2'} eq 'Y' ) {
            bccpbmod::writeFile(
                $jcl, $compile_1, "
//          DD  DISP=SHR,DSN=DB2.$db2sys.SDSNEXIT
//          DD  DISP=SHR,DSN=DB2.$db2sys.SDSNLOAD
"
            );
        }

        if ( $CCattr{'CICS'} eq 'Y' ) {
            bccpbmod::writeFile(
                $jcl, $compile_1, "
//          DD  DISP=SHR,DSN=PROD.LOAD
"
            );
        }


        if ( $CCattr{'DB2'} eq 'Y' ) {
            bccpbmod::writeFile(
                $jcl, $compile_1, "
//          DD  DISP=SHR,DSN=SYS1.SYSEXEC(INDB2TSO)
"
            );
        }

        if ( $CCattr{'MQ'} eq 'Y' ) {
            bccpbmod::writeFile(
                $jcl, $compile_1, "
//          DD  DISP=SHR,DSN=SYS1.SYSEXEC(INMQBTCH)
"
            );
        }

        bccpbmod::writeFile(
            $jcl, $compile_1, "
//SYSLMOD  DD DSN=$loadlib($mem),
//            DISP=SHR
//SYSUT1   DD UNIT=VIO,SPACE=(CYL,(2,3))
//SYSPRINT DD $sysout
//            $sysout1
"
        );
##############################################################
#
# This is just an example specific site standards and naming conventions
# must be used in addition this is an optional step this step allows for the load module to be deployed
# from the ClearCase repository to Z/OS at a later time, this load module couls also be versioned within
# ClearCase to be used for backout purposes this is an optional step. This step prepares the load module
# to be transferred to ClearCase
#
# 

    if ( $CCattr{'BT'} eq 'Y' ) {    # test 
        bccpbmod::writeFile(
            $jcl, $compile_1, "
//BTTRNLD  EXEC PGM=IKJEFT01
//SYSPRINT DD $sysout
//            $sysout1
//INDD     DD DSN=PSBH.PGMTEST,DISP=SHR
//OUTDD    DD DSN=$name.LDPREP,DISP=SHR
//SYSTSPRT DD $sysout
//            $sysout1
//SYSTSIN  DD *
 XMIT A.A DDNAME(INDD) MEMBER($mem) OUTDDNAME(OUTDD) NOLOG
"
        );
    }                                 # test 

##############################################################
#
# This is just an example specific site standards and naming conventions
# must be used in addition this is an optional step this step allows for the load module to be deployed
# from the ClearCase repository to Z/OS at a later time, this load module couls also be versioned within
# ClearCase to be used for backout purposes this is an optional step. This step actually transfers the load module
# to ClearCase
#
    if ( $CCattr{'BT'} eq 'Y' ) {     # test 
    bccpbmod::writeFile(
        $jcl, $compile_1, "
//XFERLOD  EXEC PGM=IEFBR14
//TEMP     DD DSN=$name.LDPREP,RCCEXT=LOD,DISP=OLD
"
    );

    }                                  # test  

##############################################################
#
# This is just an example specific site standards and naming conventions
# This will move a DBRM that has changed specific to the load module gnerated into the proper DBRMLIB based
# on where in the life cycle it has been changed.
#

    if ( $CCattr{'DB2'} eq 'Y' ) {
        bccpbmod::writeFile( $jcl, $compile_1, "
//* **********************************************  CAPTURE DBRM SOURCE
//DBRMSRC  EXEC PGM=IEBGENER
//SYSIN    DD  DUMMY
//SYSPRINT DD  SYSOUT=*
//SYSUT1   DD  DSN=$name.DBRMLIB($mem),DISP=OLD
//SYSUT2   DD  DSN=$dbrmlib($mem),DISP=OLD
"
        );
     }


##############################################################
#
# This is just an example specific site standards and naming conventions and DB2 Bind processes
# 
# In this example a CLIST is used to find the apporopriate DB2 system name base on where in the life cycle
# the bind is to occur the clist is not provided each site will have its own procedure for this bind process
# It is offered here a an example of how to accomplish executing a DB2 Bind, It is not limited to DB2 other 
# database systems can aslo be adapted.
# ddnames are: the $name varible has the following format
# HLQ.userid.*
#
    if ( $CCattr{'DB2'} eq 'Y' ) {

       bccpbmod::writeFile( $jcl, $compile_1, "
//* *****************************************************  BIND PACKAGE
//BIND    EXEC PGM=IKJEFT01,DYNAMNBR=20
//STEPLIB  DD  DSN=SYS3.DB2.TEST.DSNLOAD,DISP=SHR
//SYSPROC  DD  DSN=PS.ISPCLIB,DISP=SHR
//         DD  DSN=SYS3.CLIST,DISP=SHR
//SYSTSPRT DD  $sysout
//             $sysout1
//SYSPRINT DD  $sysout
//             $sysout1
//SYSUDUMP DD  SYSOUT=*
//SYSTSIN  DD  *
  FREE DDNAME(SYSPROC)
  ALLOC DDNAME(SYSPROC) DSN('PS.ISPCLIB' 'SYS3.CLIST') SHR
  \%PCODSN SYSTEM($db2sys)
  BIND PACKAGE(USKYDISDB2.$db2_owner) +
       LIB('$dbrmlib') +
       MEMBER($mem) +
       OWNER($db2_owner) QUALIFIER($db2_owner) +
       VALIDATE(BIND) +
       ISOLATION(CS) +
       RELEASE(COMMIT) +
       DEGREE(1) +
       CURRENTDATA(NO) +
       SQLERROR(NOPACKAGE) +
       EXPLAIN(NO) +
       FLAG(I) +
       ACTION(REPLACE)
  END
"
         );
     }
#
# Execute the above step and check return code, the rccbuild command exeacutes the step
#  
         my $cmd =
 "$bccpbmod::rccbuild -h $os390_hostname\@$os390_port -b db2bind -ft $jcl -i ..$delimiter..$delimiter$filename -n 4 -c LE $rccVerbosity -P " . $mfid . "_". $mem ."_";

         $status = bccpbmod::rcc_system($compile_1, $cmd);
         printf( "build Status was (%d)\n", $status ) if $debug;
         

     return $status;
}
